package com.nokia.common.gpload;

import com.nokia.common.gpload.entity.GploadResult;
import com.xxl.job.core.context.XxlJobHelper;
import lombok.extern.slf4j.Slf4j;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

@Slf4j
public class GploadUtil {
    public static final Pattern PATTERN = Pattern.compile("= (\\d+)");
    private static Process process = null;
    private static BufferedReader reader = null;

    public static GploadResult gpload(String gploadCommand) {
        GploadResult result = new GploadResult();
        result.setTaskStatus(true);
        try {
            process = Runtime.getRuntime().exec(gploadCommand);
            reader = new BufferedReader(new InputStreamReader(process.getInputStream()));
            List<String> lines = new ArrayList<>();
            String line;
            while ((line = reader.readLine()) != null) {
                lines.add(line);
            }
            log.info("gpload result: " + lines);
            XxlJobHelper.log("gpload result: " + lines);
            for (String s : lines) {
                if (s.contains("|rows Inserted")) {
                    result.setInsertedCount(getNum(s));
                }
                if (s.contains("|rows Updated")) {
                    result.setUpdatedCount(getNum(s));
                }
                if (s.contains("|data formatting errors")) {
                    result.setErrorCount(getNum(s));
                }
                if (s.contains("ERROR") || s.contains("gpload failed")) {
                    result.setTaskStatus(false);
                }
                if (s.contains("ERROR")) {
                    result.setMessage(s);
                }
            }
        } catch (IOException e) {
            log.error("gpload发生异常: {}", e.getMessage(), e);
            XxlJobHelper.log("gpload发生异常: {}", e.getMessage(), e);
            result.setTaskStatus(false);
            result.setMessage(e.getMessage());
        } finally {
            log.debug("gpload的结果为: {}", result);
            XxlJobHelper.log("gpload的结果为: {}", result);
            destroy();
        }
        return result;
    }

    private static void destroy() {
        if (reader != null) {
            try {
                reader.close();
            } catch (IOException e) {
                log.error("close reader error: {}", e.getMessage(), e);
                XxlJobHelper.log("close reader error: {}", e.getMessage(), e);
            }
            reader = null;
        }
        if (process != null) {
            process.destroy();
            process = null;
        }
    }

    /**
     * 获取gpload统计条数
     */
    private static int getNum(String s) {
        Matcher matcher = PATTERN.matcher(s);
        if (matcher.find()) {
            return Integer.parseInt(matcher.group(1));
        }

        return 0;
    }
}
